@@ -3,7 +3,7 @@
  *
  *  Authored By Chris Lord <chris@o-hand.com>
  *
- *  Copyright (c) 2005 OpenedHand Ltd - http://o-hand.com
+ *  Copyright (c) 2005, 2008 OpenedHand Ltd - http://o-hand.com
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -34,30 +34,61 @@
 #include "contacts-edit-pane.h"
 #include "contacts-main.h"
 
+void
+contacts_chooser_entry_changed_cb (GtkWidget *entry, ContactsData *data)
+{
+	const gchar *text;
+
+	text = gtk_entry_get_text (GTK_ENTRY (entry));
+	gtk_widget_set_sensitive (data->ui->chooser_add_button,
+			(text && strlen (text)));
+}
+
 void
 contacts_chooser_add_cb (GtkWidget *button, ContactsData *data)
 {
 	GtkWidget *treeview, *entry;
-	GtkListStore *model;
+	GtkTreeModel *model;
 	GtkTreeIter iter;
 	const gchar *text;
-	
+	gchar *found;
+	gboolean valid;
+
 	entry = data->ui->chooser_entry;
 	text = gtk_entry_get_text (GTK_ENTRY (entry));
 	
 	if (g_utf8_strlen (text, -1) <= 0)
 		return;
-	
-	treeview = data->ui->chooser_treeview;
-	model = GTK_LIST_STORE (
-		gtk_tree_view_get_model (GTK_TREE_VIEW (treeview)));
-	
-	gtk_list_store_append (model, &iter);
-	gtk_list_store_set (model, &iter, CHOOSER_TICK_COL, TRUE,
-			    CHOOSER_NAME_COL, text, -1);
-	
-	gtk_entry_set_text (GTK_ENTRY (entry), "");
 
+	treeview = data->ui->chooser_treeview;
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
+
+	valid = gtk_tree_model_get_iter_first (model, &iter);
+
+	/* check if the item already exists in the list and tick it if it does
+	 */
+	while (valid)
+	{
+		gtk_tree_model_get (model, &iter, CHOOSER_NAME_COL, &found, -1);
+		if (!g_ascii_strcasecmp (found, text))
+		{
+			gtk_list_store_set (GTK_LIST_STORE (model), &iter,
+					CHOOSER_TICK_COL, TRUE, -1);
+			g_free (found);
+			gtk_entry_set_text (GTK_ENTRY (entry), "");
+			contacts_ui_update_groups_list (data);
+			return;
+		}
+		g_free (found);
+		valid = gtk_tree_model_iter_next (model, &iter);
+	}
+
+	gtk_list_store_append (GTK_LIST_STORE (model), &iter);
+	gtk_list_store_set (GTK_LIST_STORE (model), &iter,
+			CHOOSER_TICK_COL, TRUE,
+			CHOOSER_NAME_COL, text, -1);
+
+	gtk_entry_set_text (GTK_ENTRY (entry), "");
 	contacts_ui_update_groups_list (data);
 }
 
@@ -157,8 +188,11 @@ contacts_delete_cb (GtkWidget *source, ContactsData *data)
 		if (!contact) return;
 
 		name = e_contact_get_const (contact, E_CONTACT_FULL_NAME);
-		if ((!name) || (g_utf8_strlen (name, 4) <= 0))
-			name = _("Unnamed");
+		if ((!name) || (g_utf8_strlen (name, 4) <= 0)) {
+			name = e_contact_get_const (contact, E_CONTACT_ORG);
+			if ((!name) || (g_utf8_strlen (name, 4) <= 0))
+				name = _("Unnamed");
+		}
 
 		contact_list = g_list_prepend (contact_list, e_contact_get (contact, E_CONTACT_UID));
 
@@ -208,12 +242,13 @@ contacts_delete_cb (GtkWidget *source, ContactsData *data)
 	g_list_free (widgets);
 }
 
-void
+int
 contacts_import (ContactsData *data, const gchar *filename, gboolean do_confirm)
 {
 	gchar *vcard_string, *card;
 	gchar **vcard_array = NULL;
-	gint i = 0;
+	gint i = 0, import_count = 0;
+	GtkWidget *dialog, *main_window = data->ui->main_window;
 #ifdef HAVE_GNOMEVFS
 	int size;
 	GnomeVFSResult vfs_result;
@@ -225,26 +260,27 @@ contacts_import (ContactsData *data, const gchar *filename, gboolean do_confirm)
 	if (g_file_get_contents (
 		filename, &vcard_string, NULL, NULL)) {
 #endif
-		vcard_array = g_strsplit (vcard_string, "END:VCARD\n", 0);
+		vcard_array = g_strsplit (vcard_string, "END:VCARD", 0);
 		while ((card = vcard_array[i++]))
 		{
 			EContact *contact = NULL;
-			gchar *str1;
+			gchar *str1, *str2;
 			/* make sure we haven't reached the end */
 			if (strlen (card) < 1)
 				continue;
 			/* END:VCARD is stripped by strsplit, so add it again here */
 			str1 = g_strconcat (card, "END:VCARD", NULL);
-			contact = e_contact_new_from_vcard (str1);
+			/* find the start of the vcard, so evolution does not complain */
+			str2 = strstr (str1, "BEGIN:VCARD");
+			contact = e_contact_new_from_vcard (str2);
 			g_free (str1);
 
 			if (contact) {
 				gint result = GTK_RESPONSE_YES;
+				import_count++;
 				if (do_confirm) {
-					GtkWidget *dialog, *main_window;
 					GList *widgets;
 					
-					main_window = data->ui->main_window;
 					dialog = gtk_message_dialog_new (
 						GTK_WINDOW (main_window),
 						0, GTK_MESSAGE_QUESTION,
@@ -271,7 +307,7 @@ contacts_import (ContactsData *data, const gchar *filename, gboolean do_confirm)
 					/* TODO: add progress indicator and callback here */
 					e_book_async_add_contact (data->book, contact,
 								   NULL, NULL);
-					/* Maually trigger the added callback so that
+					/* Manually trigger the added callback so that
 					 * the contact can be selected.
 					 */
 					#if 0
@@ -299,6 +335,7 @@ contacts_import (ContactsData *data, const gchar *filename, gboolean do_confirm)
 			filename, gnome_vfs_result_to_string (vfs_result));
 	}
 #endif
+	return import_count;
 }
 
 void
@@ -308,6 +345,7 @@ contacts_import_cb (GtkWidget *source, ContactsData *data)
 	GtkFileFilter *filter;
 	GtkWidget *main_window =
 		data->ui->main_window;
+	int import_count = 0;
 	GtkWidget *dialog = gtk_file_chooser_dialog_new (
 		_("Import Contact"),
 		GTK_WINDOW (main_window),
@@ -330,7 +368,7 @@ contacts_import_cb (GtkWidget *source, ContactsData *data)
 		gchar *filename = gtk_file_chooser_get_filename 
 					(GTK_FILE_CHOOSER (dialog));
 		if (filename) {
-			contacts_import (data, filename, FALSE);
+			import_count = contacts_import (data, filename, FALSE);
 			g_free (filename);
 		}
 	}
@@ -338,6 +376,14 @@ contacts_import_cb (GtkWidget *source, ContactsData *data)
 	contacts_set_widgets_sensitive (widgets);
 	gtk_widget_destroy (dialog);
 	g_list_free (widgets);
+
+	dialog = gtk_message_dialog_new (GTK_WINDOW (main_window),
+					0, GTK_MESSAGE_INFO,
+					GTK_BUTTONS_OK,
+					ngettext ("Imported one contact", "Imported %d contacts", import_count),
+					import_count);
+	gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
 }
 
 void
@@ -587,12 +633,11 @@ contacts_about_cb (GtkWidget *parent)
 	const char *translator_credits = _("translator-credits");
 
 	gtk_show_about_dialog (GTK_WINDOW (parent),
-		"name", GETTEXT_PACKAGE,
 		"version", VERSION,
 		"authors", authors,
 		"logo-icon-name", "contacts",
-		"website", "http://projects.o-hand.com/contacts/",
-		"copyright", "\302\251 2006 OpenedHand Ltd",
+		"website", "http://www.pimlico-project.org/",
+		"copyright", "\302\251 2006-2008 OpenedHand Ltd",
 		"translator-credits", translator_credits,
 		"license", license,
 		"wrap-license", TRUE,
@@ -633,6 +678,8 @@ contacts_is_row_visible_cb (GtkTreeModel * model, GtkTreeIter * iter,
 	if (data->selected_group && strcmp (_("All"), data->selected_group))
 	{
 		groups = e_contact_get (hash->contact, E_CONTACT_CATEGORY_LIST);
+		if (!groups && !strcmp (NO_CATEGORY_LABEL, data->selected_group))
+			result = TRUE;
 		if ((group = data->selected_group)) {
 			for (g = groups; g; g = g->next) {
 				if (strcmp (group, g->data) == 0)
@@ -662,13 +709,15 @@ contacts_is_row_visible_cb (GtkTreeModel * model, GtkTreeIter * iter,
 			gtk_tree_model_get (model, iter, CONTACT_NAME_COL,
 				&name_string, -1);
 			if (name_string) {
-				gunichar *isearch =
-				    kozo_utf8_strcasestrip (search_string);
-				if (!kozo_utf8_strstrcasestrip
-				    (name_string, isearch))
+				gchar *needle, *haystack;
+				needle = g_utf8_casefold (search_string, -1);
+				haystack = g_utf8_casefold (name_string, -1);
+				if (!strstr (haystack, needle))
 					result = FALSE;
+
+				g_free (needle);
+				g_free (haystack);
 				g_free (name_string);
-				g_free (isearch);
 			}
 		}
 	} else if ((data->ui->symbols_radiobutton)){
