@@ -24,6 +24,7 @@
 
 void contacts_search_changed_cb (GtkWidget *search_entry, ContactsData *data);
 
+void contacts_chooser_entry_changed_cb (GtkWidget *entry, ContactsData *data);
 void contacts_chooser_add_cb (GtkWidget *button, ContactsData *data);
 
 void contacts_chooser_toggle_cb (GtkCellRendererToggle * cell, gchar * path_string, gpointer user_data);
@@ -39,7 +40,7 @@ void contacts_treeview_edit_cb (GtkTreeView *treeview, GtkTreePath *arg1,
 	
 void contacts_delete_cb (GtkWidget *source, ContactsData *data);
 
-void contacts_import (ContactsData *data, const gchar *filename,
+int contacts_import (ContactsData *data, const gchar *filename,
 		      gboolean do_confirm);
 
 void contacts_import_cb (GtkWidget *source, ContactsData *data);
