@@ -2,6 +2,7 @@
  *  Contacts - A small libebook-based address book.
  *
  *  Authored By Chris Lord <chris@o-hand.com>
+ *              Thomas Wood <thomas@o-hand.com>
  *
  *  Copyright (c) 2005 OpenedHand Ltd - http://o-hand.com
  *
@@ -22,6 +23,7 @@
 #include <glib/gprintf.h>
 #include <gtk/gtk.h>
 #include <libebook/e-book.h>
+#include <libebook/e-name-western.h>
 
 #include "contacts-defs.h"
 #include "contacts-main.h"
@@ -205,13 +207,13 @@ contacts_type_entry_changed (GtkWidget *widget, EContactTypeChangeData *data)
 	
 	if (v) {
 		guint i;
-		const gchar **types =
+		const TypeTuple *types =
 			contacts_get_field_types (data->attr_name);
 		const gchar *type = (const gchar *)v->data;
 		gboolean custom_type = TRUE;
 		
-		for (i = 0; types[i]; i++) {
-			if (g_ascii_strcasecmp (type, types[i]) == 0)
+		for (i = 0; types[i].index; i++) {
+			if (g_ascii_strcasecmp (type, types[i].index) == 0)
 				custom_type = FALSE;
 		}
 		
@@ -225,6 +227,9 @@ contacts_type_entry_changed (GtkWidget *widget, EContactTypeChangeData *data)
 		} else
 			e_vcard_attribute_param_add_value (
 				data->param, (const char *)v->data);
+
+
+
 		g_list_foreach (v, (GFunc)g_free, NULL);
 		g_list_free (v);
 		*data->changed = TRUE;
@@ -249,6 +254,45 @@ contacts_entry_changed (GtkWidget *widget, EContactChangeData *data)
 		e_vcard_attribute_add_value (data->attr,
 					     (const gchar *)v->data);
 	}
+
+	if (!g_ascii_strcasecmp (e_vcard_attribute_get_name (data->attr), EVC_FN))
+	{
+		/* add N and X-EVOLUTION-FILE-AS attributes */
+		ENameWestern *name;
+		gchar *file_as;
+		EVCardAttribute *attr;
+		EVCard *evc = E_VCARD (data->contact);
+
+		name = e_name_western_parse ((char*)values->data);
+		
+		/* Add "N" attribute */
+		attr = e_vcard_get_attribute (evc, EVC_N);
+		if (attr)
+			e_vcard_attribute_remove_values (attr);
+		else
+		{
+			attr = e_vcard_attribute_new ("", EVC_N);
+			e_vcard_add_attribute (evc, attr);
+		}
+#define SAFESTR(x) (x) ? x : ""
+		e_vcard_attribute_add_value (attr, SAFESTR (name->last));
+		e_vcard_attribute_add_value (attr, SAFESTR (name->first));
+		e_vcard_attribute_add_value (attr, SAFESTR (name->middle));
+		e_vcard_attribute_add_value (attr, SAFESTR (name->prefix));
+		e_vcard_attribute_add_value (attr, SAFESTR (name->suffix));
+
+		/* Add file-as attribute for evolution */
+		file_as = g_strdup_printf ("%s, %s", name->last, name->first);
+		attr = e_vcard_get_attribute (evc, EVC_X_FILE_AS);
+		if (attr)
+			e_vcard_remove_attribute (evc, attr);
+		attr = e_vcard_attribute_new ("", EVC_X_FILE_AS);
+		e_vcard_add_attribute_with_value (evc, attr, file_as);
+		g_free (file_as);
+
+		g_free (name);
+	}
+
 	g_list_foreach (values, (GFunc)g_free, NULL);
 	g_list_free (values);
 	*data->changed = TRUE;
@@ -258,7 +302,7 @@ static GtkWidget *
 contacts_type_edit_widget_new (EVCardAttribute *attr, gboolean multi_line,
 	gboolean *changed)
 {
-	const gchar **types;
+	const TypeTuple *types;
 
 	types = contacts_get_field_types (e_vcard_attribute_get_name (attr));
 	if (types) {
@@ -292,15 +336,15 @@ contacts_type_edit_widget_new (EVCardAttribute *attr, gboolean multi_line,
 		data->attr_name = e_vcard_attribute_get_name (attr);
 		data->changed = changed;
 		
-		for (i = 0; types[i]; i++) {
+		for (i = 0; types[i].index; i++) {
 			gtk_combo_box_append_text (
-				GTK_COMBO_BOX (combo), types[i]);
+				GTK_COMBO_BOX (combo), _(types[i].value));
 			/* Note: We use a case-insensitive search here, as
 			 * specified in the spec (as the types are predefined,
 			 * we can use non-locale-friendly strcasecmp)
 			 */
 			if (first_type) {
-				if (g_ascii_strcasecmp (types[i], first_type) == 0) {
+				if (g_ascii_strcasecmp (types[i].index, first_type) == 0) {
 					first_type = NULL;
 					gtk_combo_box_set_active (
 						GTK_COMBO_BOX (combo), i);
@@ -314,9 +358,8 @@ contacts_type_edit_widget_new (EVCardAttribute *attr, gboolean multi_line,
 				(const gchar *)(first_type+2));
 			first_type = NULL;
 		}
-		gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Other"));
 		if (first_type)
-			gtk_combo_box_set_active (GTK_COMBO_BOX (combo), i);
+			gtk_combo_box_set_active (GTK_COMBO_BOX (combo), i-1);
 		gtk_widget_show (combo);
 		if (/*(e_vcard_attribute_is_single_valued (attr)) &&*/
 		    (multi_line == FALSE))
@@ -457,6 +500,9 @@ contacts_edit_widget_new (EContact *contact, EVCardAttribute *attr,
 			if (sfield) {
 				gchar *label_markup;
 
+				if (sfield->priority == 0)
+					continue;
+
 				label = gtk_label_new (NULL);
 				multiline = sfield->multiline;				
 				gtk_label_set_use_markup (GTK_LABEL (label),
@@ -500,6 +546,7 @@ contacts_edit_widget_new (EContact *contact, EVCardAttribute *attr,
 				gtk_text_buffer_set_text (buffer, string ?
 							   string : "", -1);
 				gtk_text_view_set_editable (view, TRUE);
+				gtk_text_view_set_accepts_tab (view, FALSE);
 				
 				entry = gtk_frame_new (NULL);
 				gtk_frame_set_shadow_type (GTK_FRAME (entry),
@@ -546,6 +593,7 @@ contacts_edit_widget_new (EContact *contact, EVCardAttribute *attr,
 		gtk_widget_set_name (GTK_WIDGET (view), attr_name);
 		gtk_text_buffer_set_text (buffer, string ? string : "", -1);
 		gtk_text_view_set_editable (view, TRUE);
+		gtk_text_view_set_accepts_tab (view, FALSE);
 		g_free (string);
 
 		container = gtk_frame_new (NULL);
@@ -897,30 +945,6 @@ contacts_remove_entries (GtkWidget *entry)
 	}
 }
 
-static gboolean
-contacts_widget_is_label (GtkWidget *widget)
-{
-	if (GTK_IS_LABEL (widget))
-		return TRUE;
-	else if (GTK_IS_EXPANDER (widget))
-		return FALSE;
-	else if (GTK_IS_CONTAINER (widget)) {
-		GList *c, *children =
-			gtk_container_get_children (GTK_CONTAINER (widget));
-		
-		for (c = children; c; c = c->next) {
-			if (contacts_widget_is_label (GTK_WIDGET (c->data))) {
-				g_list_free (children);
-				return TRUE;
-			}
-		}
-		
-		g_list_free (children);
-	}
-	
-	return FALSE;
-}
-
 void
 contacts_edit_set_focus_cb (GtkWidget *button, GtkWidget *widget,
 	GtkWindow *window)
