@@ -76,14 +76,30 @@ groups_combobox_changed_cb (GtkWidget *widget, ContactsData *data)
 	contacts_update_treeview (data);
 }
 
+void
+add_to_list (gpointer data, gpointer list)
+{
+	GtkTreeIter iter;
+	gtk_list_store_insert_with_values (list, &iter, -1, 0, data, -1);
+}
+
+gint
+groups_compare (const gchar *a, const gchar *b)
+{
+	gint result;
+	gchar *str1, *str2;
+
+	str1 = g_utf8_casefold(a, -1);
+	str2 = g_utf8_casefold(b, -1);
+	result = g_utf8_collate(str1, str2);
+	g_free(str1);
+	g_free(str2);
+	return result;
+}
+
 void
 contacts_ui_update_groups_list (ContactsData *data)
 {
-	void add_to_list (gpointer data, gpointer list)
-	{
-		GtkTreeIter iter;
-		gtk_list_store_insert_with_values (list, &iter, 1, 0, data, -1);
-	}
 	GtkListStore *list;
 	GtkTreeIter dummy;
 
@@ -91,13 +107,34 @@ contacts_ui_update_groups_list (ContactsData *data)
 	if (!list) return;
 	gtk_list_store_clear (list);
 	gtk_list_store_insert_with_values (list, &dummy, 0, 0, _("All"), -1);
+	gtk_list_store_insert_with_values (list, &dummy, 1, 0, NULL, -1);
+	data->contacts_groups = g_list_sort (data->contacts_groups, (GCompareFunc) groups_compare);
 	g_list_foreach (data->contacts_groups, add_to_list, list);
+	gtk_list_store_insert_with_values (list, &dummy, -1, 0, NULL, -1);
+	gtk_list_store_insert_with_values (list, &dummy, -1, 0, NO_CATEGORY_LABEL, -1);
 
 	/* Select 'All' in the groups combobox if nothing selected */
 	if (gtk_combo_box_get_active (GTK_COMBO_BOX (groups_combobox)))
 		gtk_combo_box_set_active (GTK_COMBO_BOX (groups_combobox), 0);
 }
 
+gboolean
+groups_combo_seperator_func (GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
+{
+	gchar *tmp;
+
+	gtk_tree_model_get (model, iter, 0, &tmp, -1);
+	if (!tmp)
+	{
+		return TRUE;
+	}
+	else
+	{
+		g_free (tmp);
+		return FALSE;
+	}
+}
+
 void
 create_main_window (ContactsData *data)
 {
@@ -143,7 +180,9 @@ create_main_window (ContactsData *data)
 	GtkWidget *vbox3;
 	GtkWidget *summary_vbox;
 	GtkWidget *preview_header_hbox;
+	GtkWidget *preview_namegroup_vbox;
 	GtkWidget *summary_name_label;
+	GtkWidget *summary_group_label;
 	GtkWidget *photo_image;
 	GtkWidget *scrolledwindow3;
 	GtkWidget *viewport1;
@@ -184,7 +223,7 @@ create_main_window (ContactsData *data)
 	main_menubar = gtk_menu_bar_new ();
 	gtk_box_pack_start (GTK_BOX (vbox7), main_menubar, FALSE, FALSE, 0);
 
-	contacts_menu = gtk_menu_item_new_with_mnemonic (_("_Contacts"));
+	contacts_menu = gtk_menu_item_new_with_mnemonic (_("C_ontacts"));
 	gtk_container_add (GTK_CONTAINER (main_menubar), contacts_menu);
 
 	contacts_menu_menu = gtk_menu_new ();
@@ -211,7 +250,7 @@ create_main_window (ContactsData *data)
 	contacts_quit = gtk_image_menu_item_new_from_stock ("gtk-quit", accel_group);
 	gtk_container_add (GTK_CONTAINER (contacts_menu_menu), contacts_quit);
 
-	contact_menu = gtk_menu_item_new_with_mnemonic (_("_Contact"));
+	contact_menu = gtk_menu_item_new_with_mnemonic (_("C_ontact"));
 	gtk_container_add (GTK_CONTAINER (main_menubar), contact_menu);
 	g_object_set (G_OBJECT (contact_menu), "no-show-all", TRUE, NULL);
 
@@ -234,7 +273,7 @@ create_main_window (ContactsData *data)
 	contact_quit = gtk_image_menu_item_new_from_stock ("gtk-quit", accel_group);
 	gtk_container_add (GTK_CONTAINER (contact_menu_menu), contact_quit);
 
-	edit_menu = gtk_menu_item_new_with_mnemonic (_("_Edit"));
+	edit_menu = gtk_menu_item_new_with_mnemonic (_("Ed_it"));
 	gtk_container_add (GTK_CONTAINER (main_menubar), edit_menu);
 
 	menuitem5_menu = gtk_menu_new ();
@@ -274,6 +313,8 @@ create_main_window (ContactsData *data)
 	groups_combobox = gtk_combo_box_new ();
 	GtkListStore *ls = gtk_list_store_new (1, G_TYPE_STRING);
 	GtkCellRenderer *renderer = gtk_cell_renderer_text_new ();
+	gtk_combo_box_set_row_separator_func (GTK_COMBO_BOX (groups_combobox),
+	    groups_combo_seperator_func, NULL, NULL);
 	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (groups_combobox), renderer, TRUE);
 	gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (groups_combobox), renderer, "text", 0);
 	gtk_combo_box_set_model (GTK_COMBO_BOX (groups_combobox), GTK_TREE_MODEL(ls));
@@ -346,8 +387,10 @@ create_main_window (ContactsData *data)
 	preview_header_hbox = gtk_hbox_new (FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (summary_vbox), preview_header_hbox, FALSE, TRUE, 0);
 
+	preview_namegroup_vbox = gtk_vbox_new (FALSE, 0);
+
 	summary_name_label = gtk_label_new (NULL);
-	gtk_box_pack_start (GTK_BOX (preview_header_hbox), summary_name_label, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (preview_namegroup_vbox), summary_name_label, TRUE, TRUE, 0);
 	GTK_WIDGET_SET_FLAGS (summary_name_label, GTK_CAN_FOCUS);
 	gtk_label_set_use_markup (GTK_LABEL (summary_name_label), TRUE);
 	gtk_label_set_selectable (GTK_LABEL (summary_name_label), TRUE);
@@ -355,7 +398,21 @@ create_main_window (ContactsData *data)
 	gtk_misc_set_padding (GTK_MISC (summary_name_label), 6, 0);
 	gtk_label_set_ellipsize (GTK_LABEL (summary_name_label), PANGO_ELLIPSIZE_END);
 
-	photo_image = gtk_image_new ();
+	summary_group_label = gtk_label_new (NULL);
+	gtk_box_pack_start (GTK_BOX (preview_namegroup_vbox), summary_group_label, TRUE, TRUE, 0);
+	GTK_WIDGET_SET_FLAGS (summary_group_label, GTK_CAN_FOCUS);
+	gtk_label_set_use_markup (GTK_LABEL (summary_group_label), TRUE);
+	gtk_label_set_selectable (GTK_LABEL (summary_group_label), TRUE);
+	gtk_misc_set_alignment (GTK_MISC (summary_group_label), 0, 0.5);
+	gtk_misc_set_padding (GTK_MISC (summary_group_label), 6, 0);
+	gtk_label_set_ellipsize (GTK_LABEL (summary_group_label), PANGO_ELLIPSIZE_END);
+
+	gtk_box_pack_start (GTK_BOX (preview_header_hbox), preview_namegroup_vbox, TRUE, TRUE, 0);
+
+
+	/* load stock_person as the default icon so the image has the correct
+	 * size before a contact is loaded */
+	photo_image = gtk_image_new_from_icon_name ("stock_person", GTK_ICON_SIZE_DIALOG);
 	gtk_box_pack_end (GTK_BOX (preview_header_hbox), photo_image, FALSE, TRUE, 6);
 	gtk_misc_set_padding (GTK_MISC (photo_image), 1, 0);
 
@@ -426,7 +483,7 @@ create_main_window (ContactsData *data)
 	GTK_WIDGET_SET_FLAGS (edit_done_button, GTK_CAN_DEFAULT);
 
 
-	gtk_widget_grab_focus (search_entry);
+	gtk_widget_grab_focus (contacts_treeview);
 	gtk_widget_grab_default (edit_button);
 	gtk_window_add_accel_group (GTK_WINDOW (main_window), accel_group);
 
@@ -553,6 +610,7 @@ create_main_window (ContactsData *data)
 
 	ui->summary_hbuttonbox = summary_hbuttonbox;
 	ui->summary_name_label = summary_name_label;
+	ui->summary_group_label = summary_group_label;
 	ui->summary_table = summary_table;
 	ui->summary_vbox = summary_vbox;
 
@@ -622,11 +680,16 @@ create_chooser_dialog (ContactsData *data)
 	chooser_entry = gtk_entry_new ();
 	gtk_box_pack_start (GTK_BOX (chooser_add_hbox), chooser_entry, TRUE, TRUE, 0);
 	gtk_entry_set_activates_default (GTK_ENTRY (chooser_entry), TRUE);
+	g_signal_connect (chooser_entry, "changed",
+			G_CALLBACK (contacts_chooser_entry_changed_cb), data);
 
 	add_type_button = gtk_button_new_from_stock ("gtk-add");
 	gtk_box_pack_start (GTK_BOX (chooser_add_hbox), add_type_button, FALSE, FALSE, 0);
 	GTK_WIDGET_SET_FLAGS (add_type_button, GTK_CAN_DEFAULT);
 	gtk_button_set_focus_on_click (GTK_BUTTON (add_type_button), FALSE);
+	/* initially insensitive, enabled when user enters text into the entry
+	 */
+	gtk_widget_set_sensitive (add_type_button, FALSE);
 
 	scrolledwindow5 = gtk_scrolled_window_new (NULL, NULL);
 	gtk_box_pack_start (GTK_BOX (vbox6), scrolledwindow5, TRUE, TRUE, 0);
@@ -676,6 +739,7 @@ create_chooser_dialog (ContactsData *data)
 				 GTK_TREE_MODEL (model));
 	g_object_unref (model);
 
+	data->ui->chooser_add_button = add_type_button;
 	data->ui->chooser_add_hbox = chooser_add_hbox;
 	data->ui->chooser_dialog = chooser_dialog;
 	data->ui->chooser_entry = chooser_entry;
