@@ -36,6 +36,8 @@
 #include "contacts-edit-pane.h"
 #include "contacts-ui.h"
 
+#include "contacts-dbus.h"
+
 void
 contacts_update_treeview (ContactsData *data)
 {
@@ -80,6 +82,18 @@ start_query (EBook *book, EBookStatus status, EBookView *book_view,
 		
 		e_book_view_start (book_view);
 	} else {
+		GtkWidget *dlg;
+		dlg = gtk_message_dialog_new (
+				GTK_WINDOW (data->ui->main_window),
+				0, GTK_MESSAGE_ERROR,
+				GTK_BUTTONS_OK,
+				_("Error starting address book"));
+		gtk_message_dialog_format_secondary_text (
+			GTK_MESSAGE_DIALOG (dlg),
+			_("Got error %d when getting book view"), status);
+		gtk_dialog_run (GTK_DIALOG (dlg));
+		gtk_widget_destroy (dlg);
+
 		g_warning("Got error %d when getting book view", status);
 	}
 }
@@ -96,7 +110,22 @@ opened_book (EBook *book, EBookStatus status, gpointer closure)
 			-1, start_query, data);
 		e_book_query_unref (query);
 	} else {
+		GtkWidget *dlg;
+		dlg = gtk_message_dialog_new (
+				GTK_WINDOW (data->ui->main_window),
+				0, GTK_MESSAGE_ERROR,
+				GTK_BUTTONS_OK,
+				_("Error opening address book"));
+		gtk_message_dialog_format_secondary_text (
+			GTK_MESSAGE_DIALOG (dlg),
+			_("Got error %d when opening book"), status);
+		gtk_dialog_run (GTK_DIALOG (dlg));
+		gtk_widget_destroy (dlg);
+
+
 		g_warning("Got error %d when opening book", status);
+
+
 	}
 }
 
@@ -136,8 +165,8 @@ main (int argc, char **argv)
 	GError *error = NULL;
 	BaconMessageConnection *mc;
 	ContactsData *data;	/* Variable for passing around data -
-					 * see contacts-defs.h.
-					 */
+				 * see contacts-defs.h.
+				 */
 	GOptionContext *context;
 	static gint plug = 0;
 	GtkWidget *widget;
@@ -153,10 +182,23 @@ main (int argc, char **argv)
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
-	context = g_option_context_new (" - A light-weight address-book");
+	g_set_application_name (_("Contacts"));
+
+	context = g_option_context_new (N_("- A light-weight address-book"));
 	g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
 	g_option_context_add_group (context, gtk_get_option_group (TRUE));
-	g_option_context_parse (context, &argc, &argv, NULL);
+
+	/* Parse command line */
+	if (!g_option_context_parse (context, &argc, &argv, &error))
+	{
+		g_option_context_free (context);
+
+		g_print ("%s\n", error->message);
+		g_error_free (error);
+		exit (1);
+	}
+
+	error = NULL;
 
 	mc = bacon_message_connection_new ("contacts");
 	if (!bacon_message_connection_get_is_server (mc)) {
@@ -172,6 +214,8 @@ main (int argc, char **argv)
 	bacon_message_connection_set_callback (
 		mc, (BaconMessageReceivedFunc)contacts_bacon_cb, data);
 
+	contacts_dbus_init_for_session (data);
+
 	/* Set critical errors to close application */
 	//g_log_set_always_fatal (G_LOG_LEVEL_CRITICAL);
 
@@ -236,5 +280,7 @@ main (int argc, char **argv)
 	/* free various things */
 	g_free (data->ui);
 
+	bacon_message_connection_free (mc);
+
 	return 0;
 }
