@@ -1,3 +1,23 @@
+/*
+ *  Contacts - A small libebook-based address book.
+ *
+ *  Authored By Chris Lord <chris@o-hand.com>
+ *              Thomas Wood <thomas@openedhand.com>
+ *
+ *  Copyright (c) 2005-2006 OpenedHand Ltd - http://o-hand.com
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2, or (at your option)
+ *  any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ */
+
 #include <string.h>
 
 #include "contacts-main.h"
@@ -37,7 +57,7 @@ contacts_display_summary (EContact *contact, ContactsData *data)
 	const gchar *string;
 	GtkImage *photo;
 	GList *a, *groups, *attributes;
-	gchar *name_markup, *groups_text = NULL;
+	gchar *name_markup, *groups_markup, *groups_text = NULL;
 	GValue *can_focus = g_new0 (GValue, 1);
 
 	if (!E_IS_CONTACT (contact))
@@ -49,14 +69,19 @@ contacts_display_summary (EContact *contact, ContactsData *data)
 	widget = data->ui->summary_name_label;
 	string = e_contact_get_const (contact, E_CONTACT_FULL_NAME);
 	/* Only examine 4-bytes (maximum UTF-8 character width is 4 bytes?) */
-	if ((!string) || (g_utf8_strlen (string, 4) <= 0))
-		string = _("Unnamed");
+	if ((!string) || (g_utf8_strlen (string, 4) <= 0)) {
+		string = e_contact_get_const (contact, E_CONTACT_ORG);
+		if ((!string) || (g_utf8_strlen (string, 4) <= 0))
+			string = _("Unnamed");
+	}
 	groups = e_contact_get (contact, E_CONTACT_CATEGORY_LIST);
 	groups_text = contacts_string_list_as_string (groups, ", ", FALSE);
-	if (groups_text)
-		name_markup = g_markup_printf_escaped ("<big><b>%s</b></big>\n<small>%s</small>", string ? string : "", groups_text);
-	else
-		name_markup = g_markup_printf_escaped ("<big><b>%s</b></big>", string ? string : "");
+	groups_markup = g_markup_printf_escaped ("<small>%s</small>", groups_text ? groups_text : "");
+	gtk_label_set_markup (GTK_LABEL(data->ui->summary_group_label),
+			      groups_markup);
+	g_free (groups_markup);
+
+	name_markup = g_markup_printf_escaped ("<big><b>%s</b></big>", string ? string : "");
 
 	gtk_label_set_markup (GTK_LABEL (widget), name_markup);
 	if (groups) {
@@ -90,15 +115,16 @@ contacts_display_summary (EContact *contact, ContactsData *data)
 			       widget);
 	g_object_set (widget, "n-rows", 1, NULL);
 	g_object_set (widget, "n-columns", 2, NULL);
-	attributes = e_vcard_get_attributes (E_VCARD (contact));
+	attributes = g_list_copy (e_vcard_get_attributes (E_VCARD (contact)));
 	attributes = g_list_sort (attributes, (GCompareFunc) contacts_compare_attributes);
+
 	g_value_init (can_focus, G_TYPE_BOOLEAN);
 	g_value_set_boolean (can_focus, FALSE);
 	for (a = attributes; a; a = a->next) {
 		GtkWidget *name_widget, *value_widget;
 		gchar *value_text, *name_markup;
-		GList *values;
-		const gchar **types;
+		GList *values = NULL;
+		const TypeTuple *types;
 		const gchar *attr_name;
 		EVCardAttribute *attr = (EVCardAttribute *)a->data;
 		const ContactsField *field = contacts_get_contacts_field (
@@ -107,19 +133,25 @@ contacts_display_summary (EContact *contact, ContactsData *data)
 		if (!field)
 			continue;
 		
+		attr_name = e_vcard_attribute_get_name (attr);
+		/* we already have the Full Name above ... */
+		if (!strcmp (attr_name, "FN")) continue;
+
 		values = e_vcard_attribute_get_values (attr);
 		value_text =
 			contacts_string_list_as_string (values, "\n", FALSE);
 		
-		attr_name = e_vcard_attribute_get_name (attr);
-
-		/* we already have the Full Name above ... */
-		if (!strcmp (attr_name, "FN")) continue;
+		/* don't display blank fields */
+		if (!value_text || strlen (value_text) < 1)
+		{
+			g_free (value_text);
+			continue;
+		}
 
 		types = contacts_get_field_types (attr_name);
 		if (types) {
 			gchar *types_string = NULL;
-			const gchar **valid_types;
+			const TypeTuple *valid_types;
 			GList *types_list = NULL;
 			guint i;
 
@@ -133,12 +165,12 @@ contacts_display_summary (EContact *contact, ContactsData *data)
 					types_string = (gchar *)
 							(types_list->data)+2;
 				else if (valid_types) {
-					for (i = 0; valid_types[i]; i++) {
+					for (i = 0; valid_types[i].index; i++) {
 						if (g_ascii_strcasecmp (
 							types_list->data,
-							valid_types[i]) == 0) {
+							valid_types[i].index) == 0) {
 							types_string = (gchar *)
-								valid_types[i];
+								_(valid_types[i].value);
 							break;
 						}
 					}
@@ -186,6 +218,8 @@ contacts_display_summary (EContact *contact, ContactsData *data)
 
 	widget = data->ui->summary_table;
 	contacts_remove_labels_from_focus_chain (GTK_CONTAINER (widget));
+
+	g_list_free (attributes);
 }
 
 /* Helper method to set edit/delete sensitive/insensitive */
