@@ -28,127 +28,6 @@
 #include "contacts-defs.h"
 #include "contacts-main.h"
 
-/* The following functions taken from
- * http://svn.o-hand.com/repos/kozo/server/src/kozo-utf8.c
- */
- /*****************************************************************************/
-G_INLINE_FUNC char *
-e_util_unicode_get_utf8 (const char *text, gunichar * out)
-{
-	*out = g_utf8_get_char (text);
-
-	return (G_UNLIKELY (*out == (gunichar) - 1)) ? NULL :
-	    g_utf8_next_char (text);
-}
-
-static gunichar
-stripped_char (gunichar ch)
-{
-	GUnicodeType utype;
-
-	utype = g_unichar_type (ch);
-
-	switch (utype) {
-	case G_UNICODE_CONTROL:
-	case G_UNICODE_FORMAT:
-	case G_UNICODE_UNASSIGNED:
-	case G_UNICODE_COMBINING_MARK:
-		/* Ignore those */
-		return 0;
-	default:
-		/* Convert to lowercase, fall through */
-		ch = g_unichar_tolower (ch);
-	case G_UNICODE_LOWERCASE_LETTER:
-		return ch;
-	}
-}
-
-const char *
-kozo_utf8_strstrcasestrip (const char *haystack, const gunichar * needle)
-{
-	gunichar unival, sc;
-	const char *o, *p, *q;
-	int npos;
-
-	/* FIXME this is unoptimal */
-	/* TODO use Bayer-Moore algorithm, if faster */
-	o = haystack;
-	for (p = e_util_unicode_get_utf8 (o, &unival);
-	     p && unival; p = e_util_unicode_get_utf8 (p, &unival)) {
-		sc = stripped_char (unival);
-		if (sc) {
-			/* We have valid stripped char */
-			if (sc == needle[0]) {
-				q = p;
-				npos = 1;
-
-				while (needle[npos]) {
-					q = e_util_unicode_get_utf8 (q,
-								     &unival);
-					if (!q || !unival)
-						return NULL;
-
-					sc = stripped_char (unival);
-					if ((!sc) || (sc != needle[npos]))
-						break;
-
-					npos++;
-				}
-
-				if (!needle[npos])
-					return o;
-			}
-		}
-
-		o = p;
-	}
-
-	return NULL;
-}
-
-gunichar *
-kozo_utf8_strcasestrip (const char *str)
-{
-	gunichar *needle;
-	gunichar unival, sc;
-	int nlen, normlen;
-	const char *p;
-	char *norm;
-
-	/* FIXME unoptimal, too many iterations */
-	norm = g_utf8_normalize (str, -1, G_NORMALIZE_DEFAULT);
-	normlen = g_utf8_strlen (norm, -1);
-	if (G_UNLIKELY (normlen == 0)) {
-		g_free (norm);
-		return NULL;
-	}
-
-	/* Over-estimates length */
-	needle = g_new (gunichar, normlen + 1);
-
-	nlen = 0;
-	for (p = e_util_unicode_get_utf8 (norm, &unival);
-	     p && unival; p = e_util_unicode_get_utf8 (p, &unival)) {
-		sc = stripped_char (unival);
-		if (sc)
-			needle[nlen++] = sc;
-	}
-
-	g_free (norm);
-
-	/* NULL means there was illegal utf-8 sequence */
-	if (!p)
-		return NULL;
-
-	/* If everything is correct, we have decomposed, lowercase, stripped 
-	 * needle */
-	needle[nlen] = 0;
-	if (normlen == nlen)
-		return needle;
-	else
-		return g_renew (gunichar, needle, nlen + 1);
-}
-
 /******************************************************************************/
 
 /* List of always-available fields */
@@ -160,6 +39,11 @@ static const ContactsField contacts_fields[] = {
 	{ "TEL", 0, N_("Phone"), FALSE, 20, FALSE },
 	{ "EMAIL", 0, N_("Email"), FALSE, 30, FALSE },
 	{ "X-JABBER", 0, N_("Jabber"), FALSE, 35, FALSE },
+	{ "X-MSN", 0, N_("MSN"), FALSE, 35, FALSE },
+	{ "X-YAHOO", 0, N_("Yahoo"), FALSE, 35, FALSE },
+	{ "X-AIM", 0, N_("AIM"), FALSE, 35, FALSE },
+	{ "X-ICQ", 0, N_("ICQ"), FALSE, 35, FALSE },
+	{ "X-GROUPWISE", 0, N_("Groupwise"), FALSE, 35, FALSE },
 	{ "ADR", 0, N_("Address"), FALSE, 40, FALSE },
 	{ "NICKNAME", E_CONTACT_NICKNAME, NULL, FALSE, 110, TRUE },
 	{ "BDAY", E_CONTACT_BIRTH_DATE, NULL, FALSE, 120, TRUE },
@@ -171,42 +55,48 @@ static const ContactsField contacts_fields[] = {
 
 
 static const ContactsStructuredField contacts_sfields[] = {
-	{ "ADR", 0, N_("PO Box"), FALSE },
-	{ "ADR", 1, N_("Ext."), TRUE },
-	{ "ADR", 2, N_("Street"), TRUE },
-	{ "ADR", 3, N_("Locality"), FALSE },
-	{ "ADR", 4, N_("Region"), FALSE },
-	{ "ADR", 5, N_("Post Code"), FALSE },
-	{ "ADR", 6, N_("Country"), FALSE },
+	{ "ADR", 0, N_("PO Box"), FALSE, -1 },
+	{ "ADR", 1, N_("Ext."), TRUE, 0 },
+	{ "ADR", 2, N_("Street"), TRUE, 1 },
+	{ "ADR", 3, N_("City"), FALSE, 1 },
+	{ "ADR", 4, N_("Provice"), FALSE, 1 },
+	{ "ADR", 5, N_("Postal Code"), FALSE, 1 },
+	{ "ADR", 6, N_("Country"), FALSE, 1 },
 	{ NULL, 0, NULL, FALSE }
 };
 
-struct contacts_field_types_def {
-	char *field;
-	gchar **types;
+/* Translators: Home, Cell, Office, Fax and Other are types of telephone attributes */
+static const TypeTuple tel_types[] =
+{
+	{"HOME", N_("Home")},
+	{"CELL", N_("Cell")},
+	{"WORK", N_("Work")},
+	{"FAX", N_("Fax")},
+	{"OTHER", N_("Other")},
+	{NULL}
 };
 
-static const struct contacts_field_types_def contacts_field_types[] = {
-        /* TODO: can these be i18n-ized? */
-	{ "TEL", (gchar*[]){"Home", "Msg", "Work", "Pref", "Voice", "Fax",
-			    "Cell", "Video", "Pager", "BBS", "Modem", "Car",
-			    "ISDN", "PCS", NULL }},
-	{ "EMAIL", (gchar*[]){"Internet", "X400", "Pref", NULL}},
-	{ "ADR", (gchar*[]){"Dom", "Intl", "Postal", "Parcel", "Home", "Work", "Pref", NULL}},
-	{ NULL, NULL }
+/* Translators: Home, Office and Other are types of contact attributes */
+static const TypeTuple generic_types[] =
+{
+	{"HOME", N_("Home")},
+	{"WORK", N_("Work")},
+	{"OTHER", N_("Other")},
+	{NULL}
 };
 
-const gchar **
+const TypeTuple *
 contacts_get_field_types (const gchar *attr_name)
 {
-	guint i;
+	if (!attr_name)
+		return NULL;
 
-	for (i = 0; contacts_field_types[i].field; i++) {
-		if (strcmp (contacts_field_types[i].field, attr_name) == 0)
-			return (const gchar **)contacts_field_types[i].types;
-	}
-	
-	return NULL;
+	if (!strcmp (attr_name, "TEL"))
+		return tel_types;
+	else if (!strcmp (attr_name, "EMAIL") || !strcmp (attr_name, "ADR"))
+		return generic_types;
+	else
+		return NULL;
 }
 
 const ContactsStructuredField *
@@ -278,13 +168,13 @@ contacts_compare_attributes (EVCardAttribute *attrA, EVCardAttribute *attrB)
 
 	for (i = 0; contacts_fields[i].vcard_field; i++)
 	{
-		if (strcmp (nameA, contacts_fields[i].vcard_field))
+		if (a == 0 && strcmp (nameA, contacts_fields[i].vcard_field) == 0)
 			a = contacts_fields[i].priority;
-		if (strcmp (nameB, contacts_fields[i].vcard_field))
+		if (b == 0 && strcmp (nameB, contacts_fields[i].vcard_field) == 0)
 			b = contacts_fields[i].priority;
 	}
 
-	return b - a;
+	return a - b;
 }
 
 EContact *
@@ -368,6 +258,7 @@ contacts_set_selected_contact (ContactsData *data, const gchar *uid)
 		if (strcmp (uid, ruid) == 0) {
 			GtkTreeSelection *selection =
 				gtk_tree_view_get_selection (treeview);
+			gtk_tree_selection_unselect_all (selection);
 			if (selection)
 				gtk_tree_selection_select_iter (
 					selection, &iter);
@@ -445,14 +336,14 @@ contacts_load_photo (EContact *contact)
  * value. Evolution tends to add empty fields like this a lot - as does
  * contacts, to show required fields (but it removes them after editing, via
  * this function).
- * TODO: This really doesn't need to be recursive.
  */
 void
 contacts_clean_contact (EContact *contact)
 {
-	GList *attributes, *c;
+	GList *attributes, *c, *trash = NULL;
 
 	attributes = e_vcard_get_attributes (E_VCARD (contact));
+
 	for (c = attributes; c; c = c->next) {
 		EVCardAttribute *a = (EVCardAttribute*)c->data;
 		GList *values = e_vcard_attribute_get_values (a);
@@ -462,11 +353,14 @@ contacts_clean_contact (EContact *contact)
 				remove = FALSE;
 		}
 		if (remove) {
-			e_vcard_remove_attribute (E_VCARD (contact), a);
-			contacts_clean_contact (contact);
-			break;
+			trash = g_list_prepend (trash, a);
 		}
 	}
+
+	for (c = g_list_first (trash); c; c = c->next)
+		e_vcard_remove_attribute (E_VCARD (contact), c->data);
+
+	g_list_free (trash);
 }
 
 gboolean
@@ -863,3 +757,49 @@ contacts_set_widgets_sensitive (GList *widgets)
 		gtk_widget_set_sensitive (GTK_WIDGET (w->data), TRUE);
 	}
 }
+
+gboolean
+contacts_window_get_is_visible (GtkWindow *window)
+{
+	GdkWindowState  state;
+	GdkWindow      *gdk_window;
+
+	g_return_val_if_fail (GTK_IS_WINDOW (window), FALSE);
+
+	gdk_window = GTK_WIDGET (window)->window;
+	if (!gdk_window) {
+		return FALSE;
+	}
+
+	state = gdk_window_get_state (gdk_window);
+	if (state & (GDK_WINDOW_STATE_WITHDRAWN | GDK_WINDOW_STATE_ICONIFIED)) {
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+
+/* Takes care of moving the window to the current workspace. */
+void
+contacts_window_present (ContactsData *data)
+{
+	guint32 timestamp;
+	GtkWidget *window = gtk_widget_get_toplevel (data->ui->summary_vbox);
+
+	g_return_if_fail (GTK_WIDGET_TOPLEVEL (window));
+	g_return_if_fail (GTK_IS_WINDOW (window));
+
+	/* There are three cases: hidden, visible, visible on another
+	 * workspace.
+	 */
+	if (!contacts_window_get_is_visible (GTK_WINDOW (window))) {
+		/* Hide it so present brings it to the current workspace. */
+		gtk_widget_hide (GTK_WIDGET (window));
+	}
+
+	timestamp = gtk_get_current_event_time ();
+	gtk_window_set_skip_taskbar_hint (GTK_WINDOW (window), FALSE);
+	gtk_window_present_with_time (GTK_WINDOW (window), timestamp);
+}
+
